@@ -1,6 +1,8 @@
 use super::client::Client;
 use serde::{Deserialize, Serialize};
 use anyhow::{Result, anyhow};
+use reqwest::{Method, StatusCode};
+use serde_json::Value;
 use crate::harbor::common::Label;
 use super::common::Signature;
 
@@ -16,8 +18,8 @@ pub struct Tag {
     pub size: u64,
     pub signature: Option<Signature>,
     pub labels: Vec<Label>,
-    pub scan_overview: Option<serde_json::Value>,
-    pub config: Option<serde_json::Value>,
+    pub scan_overview: Option<Value>,
+    pub config: Option<Value>,
     pub push_time: String,
     pub pull_time: String,
     #[serde(rename = "os.version")]
@@ -25,28 +27,32 @@ pub struct Tag {
 }
 
 impl Client {
+    /// Get tags of a relevant repository.
     pub async fn list_tags(&self, repo_name: &str, label_id: Option<&str>, detail: Option<bool>) -> Result<Vec<Tag>> {
-        let url = self.build_api(format!("repositories/{}/tags", repo_name));
-        let req = self.client.get(url);
-        let mut query: Vec<(String, String)> = Vec::new();
+        let path = format!("/repositories/{}/tags", repo_name);
+        let mut params = Vec::new();
         if let Some(label_id) = label_id {
-            query.append(&mut vec![(String::from("label"), label_id.to_string())]);
+            params.push(("label", label_id.to_string()));
         }
         if let Some(detail) = detail {
-            query.append(&mut vec![(String::from("detail"), detail.clone().to_string())]);
+            params.push(("detail", detail.to_string()));
+        }
+        let resp = self.build_request(Method::GET, path).query(&params).send().await?;
+        if resp.status().eq(&StatusCode::OK) {
+            Ok(resp.json::<Vec<Tag>>().await?)
+        } else {
+            Err(anyhow!("failed to list tags: {}", resp.text().await?))
         }
-        let resp = req.send().await?;
-        let tags = resp.json::<Vec<Tag>>().await?;
-        Ok(tags)
     }
 
+    /// Delete a tag in a repository.
     pub async fn delete_tag(&self, repo_name: &str, tag_name: &str) -> Result<()> {
-        let url = self.build_api(format!("repositories/{}/tags/{}", repo_name, tag_name));
-        let resp = self.client.delete(url).send().await?;
-        if resp.status().is_success() {
+        let path = format!("/repositories/{}/tags/{}", repo_name, tag_name);
+        let resp = self.build_request(Method::DELETE, path).send().await?;
+        if resp.status().eq(&reqwest::StatusCode::OK) {
             Ok(())
         } else {
-            Err(anyhow!("{} {}", resp.status(), resp.text().await?))
+            Err(anyhow!("failed to delete tag: {}", resp.text().await?))
         }
     }
-}
\ No newline at end of file
+}
